package com.hcst.youteng.controller;

import com.github.qcloudsms.SmsSingleSenderResult;
import com.hcst.youteng.service.ISmsService;
import com.hcst.youteng.utils.AuthCodeUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.Map;

import static com.hcst.youteng.utils.SmsUtil.*;

@Slf4j
@RestController
@RequestMapping("/rest/sms")
public class SmsCtrl {

    @Autowired
    private RedisTemplate<String, Object> redisTemplate;
    @Autowired
    private ISmsService smsService;

    @RequestMapping(value = "/sendCode",
            method = RequestMethod.POST,
            produces = "application/json")
    @ResponseBody
    public Map<String, Object> send(@RequestBody Map<String, String> requestValue) {

        Map<String, String> repo = new HashMap<>();
        long nowTimeStamp = System.currentTimeMillis();
        boolean isSuccess = true;
        int count = 1;
        String authCode= AuthCodeUtil.getRandomCode();
        requestValue.put("authCode",authCode);
        SmsSingleSenderResult sendSmsResponse = null;
        String redisSmsKey = getRedisSmsKey(requestValue.get("sceneName"), requestValue.get("phoneNumber"));
        try {
            //发送之前如果有这个key值,直接忽略即可。
            if (getRedisSmsValue(redisTemplate, redisSmsKey).isEmpty()) {
                sendSmsResponse = smsService.sendAuthCode(requestValue);
            } else {
                Map<String, Object> redisSmsValue = getRedisSmsValue(redisTemplate, redisSmsKey);
                if ((int) redisSmsValue.get("count") > SMS_COUNT) {
                    isSuccess = false;
                    repo.put("errorMsg", "今日短信超过限制");
                } else {
                    if (nowTimeStamp - (Long) redisSmsValue.get("sendTime") < 60000) {
                        isSuccess = false;
                        repo.put("errorMsg", "发送验证码过于频繁");
                    } else {
                        sendSmsResponse = smsService.sendAuthCode(requestValue);
                        count = (int) redisSmsValue.get("count") + 1;
                    }
                }
            }
            if (isSuccess) {
                if (sendSmsResponse.result!=0) {
                    isSuccess = false;
                    repo.put("errorMsg", sendSmsResponse.errMsg);
                } else {
                    updateRedisSmsValue(redisTemplate, redisSmsKey, joinRedisData(authCode, nowTimeStamp, count, false));
                }
            }
        } catch (Exception ex) {
            isSuccess = false;
            repo.put("errorMsg", "未知错误" + ex.getMessage());
            log.error(ex.getMessage());
        }
        return smsResp(isSuccess, repo);
    }

    @RequestMapping(value = "/verifyCode",
            method = RequestMethod.POST,
            produces = "application/json")
    @ResponseBody
    public Map<String, Object> verify(@RequestBody Map<String, String> requestValue) {
        Map<String, String> repo = new HashMap<>();
        boolean isSuccess = true;
        //拼接key redis_sms_key
        String redisSmsKey = getRedisSmsKey(requestValue.get("sceneName"), requestValue.get("phoneNumber"));
        try {
            //发送之前如果有这个key值,直接忽略即可。
            Map<String, Object> redisSmsValue = getRedisSmsValue(redisTemplate, redisSmsKey);
            if (redisSmsValue.isEmpty() || (boolean) redisSmsValue.get("isVerify")
                    || ((String) redisSmsValue.get("code")).isEmpty()) {
                isSuccess = false;
                repo.put("errorMsg", "验证码失效，请重新获取");
            } else {
                String code = (String) getRedisSmsValue(redisTemplate, redisSmsKey).get("code");
                if (!code.equals(requestValue.get("code"))) {
                    isSuccess = false;
                    repo.put("errorMsg", "验证码错误，请重新验证");
                } else {
                    updateRedisSmsValue(redisTemplate, redisSmsKey, joinRedisData(code, (long) redisSmsValue.get("sendTime"), (int) redisSmsValue.get("count"), true));
                }
            }
        } catch (Exception ex) {
            log.error(ex.getMessage());
            isSuccess = false;
            repo.put("errorMsg", "未知错误" + ex.getMessage());
        }
        return smsResp(isSuccess, repo);
    }


}
